@@ -24,6 +24,7 @@
 // Debug precodes
 #define MDECODE_DPEDANTIC ((glb.debug >= MDECODE_DEBUG_PEDANTIC))
 #define MDECODE_DNORMAL   ((glb.debug >= MDECODE_DEBUG_NORMAL  ))
+#define DMD if ((glb.debug >= MDECODE_DEBUG_NORMAL))
 
 
 /* our base 64 decoder table */
@@ -213,14 +214,14 @@ int qpmode : Selects which decoding ruleset to use ( refer to RFC2047 )
 Output:        Decoded string is superimposed over the provided line parameter
 Errors:
 ------------------------------------------------------------------------*/
-int MDECODE_decode_quoted_printable( char *line, int qpmode )
+int MDECODE_decode_quoted_printable( char *line, int qpmode, char esc_char )
 {
 
 	char c;								/* The Character to output */
 	int op, ip; 						/* OutputPointer and InputPointer */
 	int slen = strlen(line); /* Length of our line */
 
-	if (MDECODE_DPEDANTIC) LOGGER_log("%s:%d:MDECODE_decode_quoted_printable:DEBUG: Input length = %d\n",FL,slen);
+	DMD LOGGER_log("%s:%d:MDECODE_decode_quoted_printable:DEBUG: input string = '%s' Input length = %d\n",FL, line, slen);
 
 	/* Initialise our "pointers" to the start of the encoded string */
 	ip=op=0;
@@ -232,7 +233,7 @@ int MDECODE_decode_quoted_printable( char *line, int qpmode )
 		c = line[ip];
 
 		/* if we have the quoted-printable esc char, then lets get cracking */
-		if (c == '=')
+		if (c == esc_char)
 		{
 
 			/* if we have another two chars... */
@@ -306,7 +307,7 @@ int MDECODE_decode_quoted_printable( char *line, int qpmode )
 		else
 			if (( c == '_' ) && ( qpmode == MDECODE_QPMODE_ISO ))
 			{
-				// RFC2047  (Section 4.2.1) says that if we encounter a '_' character in our ISO encodings then
+				// RFC2047  (Section 4.2.(2)(3)) says that if we encounter a '_' character in our ISO encodings then
 				//	we must convert that to a space ( as we are not allowed to have spaces in any
 				c = ' ';
 			}
@@ -324,7 +325,7 @@ int MDECODE_decode_quoted_printable( char *line, int qpmode )
 
 	line[op]='\0';
 
-	if (MDECODE_DPEDANTIC) LOGGER_log("%s:%d:MDECODE_decode_quoted_printable:DEBUG: Output length = %d\n", FL, strlen(line));
+	DMD LOGGER_log("%s:%d:MDECODE_decode_quoted_printable:DEBUG: Output = '%s' Output length = %d\n", FL, line, strlen(line));
 
 	// 2003-01-26:PLD: Changed from (op -1) -=> op
 	return op;
@@ -348,15 +349,19 @@ int MDECODE_decode_qp_text( char *line )
 {
 	if (glb.decode_qp == 0) return strlen(line);
 
-	return MDECODE_decode_quoted_printable( line, MDECODE_QPMODE_STD );
+	return MDECODE_decode_quoted_printable( line, MDECODE_QPMODE_STD, '=' );
 }
 
-
 int MDECODE_decode_qp_ISO( char *line )
 {
-	return MDECODE_decode_quoted_printable( line, MDECODE_QPMODE_ISO );
+//	return MDECODE_decode_quoted_printable( line, MDECODE_QPMODE_ISO, '=' );
+	return MDECODE_decode_quoted_printable( line, MDECODE_QPMODE_STD, '=' );
 }
 
+int MDECODE_decode_multipart( char *line )
+{
+	return MDECODE_decode_quoted_printable( line, MDECODE_QPMODE_STD, '%' );
+}
 
 
 
@@ -376,11 +381,8 @@ int MDECODE_decode_ISO( char *isostring, int size )
 	char encoding_charset[ MDECODE_ISO_CHARSET_SIZE_MAX ];
 	char *iso_start, *iso_end;
 	int iso_decoded;
-#ifdef ISO_DECODE_STRTOK
-	struct PLD_strtok xst;
-#endif
 
-	//LOGGER_log("%s:%d:DEBUGX: ISO-string='%s'",FL,isostring);
+	DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: ISO-string='%s'",FL,isostring);
 
 	// Process of decoding the ISO encoded string sequence.
 	//	( this process is repeated until we run out of ISO sequences )
@@ -403,27 +405,27 @@ int MDECODE_decode_ISO( char *isostring, int size )
 		iso_decoded = 0;
 
 		start_pair = strstr( isostring, "=?" );
-		if ( start_pair ) end_pair   = strstr( start_pair +2, "?=" );
+//		if ( start_pair ) end_pair   = strstr( start_pair +2, "?=" );
 
-		if (( start_pair != NULL )&&( end_pair != NULL ))
+		if (( start_pair != NULL ))
 		{
 			iso_start = start_pair;
 
 			// There's probably a better way of doing this, but, for us to find the end of this
 			//	particular 'ISO' sequence, we need to hop past 3 more ?'s ( assuming we've already
 			//	found the first one.
+			DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: ISO start = %s",FL,iso_start);
 
-			iso_end = strchr( iso_start +strlen("=?"), '?' );
-			if (iso_end) iso_end = strchr( iso_end +1, '?' );
-			if (iso_end) iso_end = strchr( iso_end +1, '?' );
-			if (iso_end) iso_end+=2;
-
-			//			iso_end = strstr( iso_start +strlen("=?"), "?=" ) +strlen("?=");
+			iso_end = strchr( iso_start +strlen("=?"), '?' ); // Jump past the encoding
+			if (iso_end) iso_end = strchr( iso_end +1, '?' ); // Jump past the Q or B
+			if (iso_end) iso_end = strpbrk( iso_end +1, "?\n\r\t;" );  // dropped the SPACE here.
+			if ((iso_end != NULL)&&(*iso_end == '?')) iso_end+=2;
 
 
 			if ( (iso_start) && (iso_end) )
 			{
 				char *token_end;
+				char restore_char='\0';
 
 				// Copy the Encoding page/code.
 				iso = iso_start +strlen("=?");
@@ -431,6 +433,7 @@ int MDECODE_decode_ISO( char *isostring, int size )
 				token_end = strchr(iso,'?');
 				if (token_end) *token_end = '\0';
 				snprintf( encoding_charset, sizeof( encoding_charset ), "%s", iso);
+				DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: ISO char set = '%s'",FL,encoding_charset);
 
 				iso = token_end +1;
 
@@ -440,60 +443,40 @@ int MDECODE_decode_ISO( char *isostring, int size )
 
 				iso = token_end +1;
 
+				DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: ISO encoding char = '%c'",FL,encoding_type);
+
 				// Get the encoded string
-				token_end = strchr(iso,'?');
-				if (token_end) *token_end = '\0';
-
-
-
-#ifdef ISO_DECODE_STRTOK
-				iso = PLD_strtok( &xst, iso_start, "?" );
-
-				if (iso)
+				token_end = strpbrk(iso,"?;\n\r\t"); //DROPPED THE SPACE here
+				if (token_end != NULL)
 				{
-					iso = PLD_strtok( &xst, NULL, "?" ); /* encoding code-page */
-					if (iso)
+					if ((*token_end != '?')&&(*token_end != ';'))
 					{
-						snprintf( encoding_charset, sizeof( encoding_charset ), "%s", iso );
+						restore_char = *token_end;
 					}
+					*token_end = '\0';
 				}
 
-				if (iso)
-				{
-					iso = PLD_strtok( &xst, NULL, "?" ); /* encoding type */
-					if (iso)
-					{
-						encoding_type = *iso;
-					}
-				}
 
-				if (iso)
-				{
-					iso = PLD_strtok( &xst, NULL,"?" ); /* encoded string */
-				}
-#endif
-/* #endif ISO_DECODE_STRTOK */
+
+
+
 
 
 				if (iso)
 				{
-					if (MDECODE_DNORMAL) LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Encoded String = '%s'\n", FL, iso );
+					DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Encoded String = '%s'\n", FL, iso );
 					switch ( encoding_type ) {
 
 						case MDECODE_ISO_ENCODING_Q:
 						case MDECODE_ISO_ENCODING_q:
-							if (MDECODE_DNORMAL) {
-								LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Decoding filename using Quoted-Printable (%s)\n", FL, iso);
-							}
+							DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Decoding filename using Quoted-Printable (%s)\n", FL, iso);
 							MDECODE_decode_qp_ISO(iso);
 							iso_decoded = 1;
 							break;
 
 						case MDECODE_ISO_ENCODING_B:
 						case MDECODE_ISO_ENCODING_b:
-							if (MDECODE_DNORMAL) {
-								LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Decoding filename using BASE64 (%s)\n", FL, iso);
-							}
+							DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Decoding filename using BASE64 (%s)\n", FL, iso);
 							MDECODE_decode_short64( iso );
 							iso_decoded = 1;
 							break;
@@ -508,7 +491,7 @@ int MDECODE_decode_ISO( char *isostring, int size )
 					{
 						char *new_end_pos;
 
-						if (MDECODE_DNORMAL) LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Decoded String = '%s'\n", FL, iso );
+						DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Decoded String = '%s'\n", FL, iso );
 						*iso_start = '\0'; // Terminate the original string before the start of the ISO data
 
 						// Because sometimes ISO strings are broken over multiple lines
@@ -518,12 +501,44 @@ int MDECODE_decode_ISO( char *isostring, int size )
 						//		=? sequence using strstr() because it might traverse 
 						//		beyond the end of the current 'line' (ie, \r\n termination)
 
-						new_end_pos = iso_end;
-						while ((*new_end_pos == ' ')||(*new_end_pos == '\t')) new_end_pos++;
-						if (strncmp(new_end_pos,"=?",2)==0) iso_end = new_end_pos;
+						if (token_end)
+						{
+							iso_end = token_end +1;
+							DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: iso_end = '%20s'",FL, iso_end);
+							while ((*iso_end == '?')||(*iso_end == '=')) iso_end++;
+							DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: iso_end = '%20s'",FL, iso_end);
+
+							new_end_pos = iso_end;
+							while ((*new_end_pos == ' ')||(*new_end_pos == '\t')) new_end_pos++;
+							if (strncmp(new_end_pos,"=?",2)==0) iso_end = new_end_pos;
+						} else {
+							iso_end = NULL;
+						}
+
+						DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: ISO-END = '%20s'",FL,iso_end);
+
+
 						
-						snprintf( iso_copy, size, "%s%s%s", isostring, iso, iso_end );
+						/** We now have the string split into 3 peices,
+						  **	isostring = pointing to the start
+						  **	iso = newly decoded string
+						  **	iso_end = start of string after the non-decoded ISO portion
+						  **/
+						
+						/** Generate new string using the decoded ISO to a temporary string **/
+						if (restore_char != '\0')
+						{
+							DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Recomposing string with restore-char of '%c'",FL,restore_char);
+							DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: ISO-end (start of end of string) is \n%s",FL,iso_end);
+							snprintf( iso_copy, size, "%s%s%c%s", isostring, iso, restore_char, (iso_end?iso_end:"") );
+						} else {
+							DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: Recomposing string with NO restore-char",FL,restore_char);
+						  	snprintf( iso_copy, size, "%s%s%s", isostring, iso, (iso_end?iso_end:"") );
+						}
+
+						/** Switch the new headers over to the original headers again **/
 						snprintf( isostring, size, "%s", iso_copy );
+						DMD LOGGER_log("%s:%d:MDECODE_decode_ISO:DEBUG: New ISO string = \n%s",FL,isostring);
 
 					}
 
