@@ -1,38 +1,38 @@
 /***************************************************************************
  * tnef2txt
-*   A program to decode application/ms-tnef MIME attachments into text
-*   for those fortunate enough not to be running either a Microsoft
-*   operating system or mailer.
-*
+ *   A program to decode application/ms-tnef MIME attachments into text
+ *   for those fortunate enough not to be running either a Microsoft
+ *   operating system or mailer.
+ *
  * 18/10/2001
-* Brutally cropped by Paul L Daniels (pldaniels@pldaniels.com) in order
-* to accommodate the needs of ripMIME/Xamime/Inflex without carrying too
-* much excess baggage.
-*
+ * Brutally cropped by Paul L Daniels (pldaniels@pldaniels.com) in order
+ * to accommodate the needs of ripMIME/Xamime/Inflex without carrying too
+ * much excess baggage.
+ *
  * Brandon Long (blong@uiuc.edu), April 1997
-* 1.0 Version
-*   Supports most types, but doesn't decode properties.  Maybe some other
-*   time.
-*
+ * 1.0 Version
+ *   Supports most types, but doesn't decode properties.  Maybe some other
+ *   time.
+ *
  * 1.1 Version (7/1/97)
-*   Supports saving of attAttachData to a file given by attAttachTitle
-*   start of property decoding support
-*
+ *   Supports saving of attAttachData to a file given by attAttachTitle
+ *   start of property decoding support
+ *
  * 1.2 Version (7/19/97)
-*   Some architectures don't like reading 16/32 bit data on unaligned
-*   boundaries.  Fixed, losing efficiency, but this doesn't really
-*   need efficiency anyways.  (Still...)
-*   Also, the #pragma pack from the MSVC include file wasn't liked
-*   by most Unix compilers, replaced with a GCCism.  This should work
-*   with GCC, but other compilers I don't know.
-*
+ *   Some architectures don't like reading 16/32 bit data on unaligned
+ *   boundaries.  Fixed, losing efficiency, but this doesn't really
+ *   need efficiency anyways.  (Still...)
+ *   Also, the #pragma pack from the MSVC include file wasn't liked
+ *   by most Unix compilers, replaced with a GCCism.  This should work
+ *   with GCC, but other compilers I don't know.
+ *
  * 1.3 Version (7/22/97)
-*   Ok, take out the DTR over the stream, now uses read_16.
-*
+ *   Ok, take out the DTR over the stream, now uses read_16.
+ *
  * NOTE: THIS SOFTWARE IS FOR YOUR PERSONAL GRATIFICATION ONLY.  I DON'T
-* IMPLY IN ANY LEGAL SENSE THAT THIS SOFTWARE DOES ANYTHING OR THAT IT WILL
-* BE USEFULL IN ANY WAY.  But, you can send me fixes to it, I don't mind.
-***************************************************************************/
+ * IMPLY IN ANY LEGAL SENSE THAT THIS SOFTWARE DOES ANYTHING OR THAT IT WILL
+ * BE USEFULL IN ANY WAY.  But, you can send me fixes to it, I don't mind.
+ ***************************************************************************/
 
 #include <stdio.h>
 #include <sys/stat.h>
@@ -72,7 +72,7 @@ struct TNEF_globals {
 	uint8 *tnef_limit;
 
 	int (*filename_decoded_report)(char *, char *);	// Pointer to our filename reporting function
-	
+
 };
 
 static struct TNEF_globals TNEF_glb;
@@ -85,19 +85,19 @@ static struct TNEF_globals TNEF_glb;
 
 
 /*-----------------------------------------------------------------\
- Function Name	: TNEF_init
- Returns Type	: int
- 	----Parameter List
-	1. void , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: TNEF_init
+  Returns Type	: int
+  ----Parameter List
+  1. void , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int TNEF_init( void )
 {
@@ -114,19 +114,19 @@ int TNEF_init( void )
 
 
 /*-----------------------------------------------------------------\
- Function Name	: TNEF_set_decode
- Returns Type	: int
- 	----Parameter List
-	1. int level , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: TNEF_set_decode
+  Returns Type	: int
+  ----Parameter List
+  1. int level , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int TNEF_set_decode( int level )
 {
@@ -166,19 +166,19 @@ int TNEF_set_verbosity( int level )
 
 
 /*-----------------------------------------------------------------\
- Function Name	: TNEF_set_verbosity_contenttype
- Returns Type	: int
- 	----Parameter List
-	1. int level , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: TNEF_set_verbosity_contenttype
+  Returns Type	: int
+  ----Parameter List
+  1. int level , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int TNEF_set_verbosity_contenttype( int level )
 {
@@ -187,20 +187,20 @@ int TNEF_set_verbosity_contenttype( int level )
 }
 
 /*-----------------------------------------------------------------\
- Function Name	: TNEF_set_filename_report_fn
- Returns Type	: int
- 	----Parameter List
-	1. int (*ptr_to_fn)(char *, 
-	2.  char *) , 
- 	------------------
- Exit Codes	: 
- Side Effects	: 
+  Function Name	: TNEF_set_filename_report_fn
+  Returns Type	: int
+  ----Parameter List
+  1. int (*ptr_to_fn)(char *, 
+  2.  char *) , 
+  ------------------
+  Exit Codes	: 
+  Side Effects	: 
+  --------------------------------------------------------------------
+Comments:
+
 --------------------------------------------------------------------
- Comments:
- 
---------------------------------------------------------------------
- Changes:
- 
+Changes:
+
 \------------------------------------------------------------------*/
 int TNEF_set_filename_report_fn( int (*ptr_to_fn)(char *, char *) )
 {
@@ -237,10 +237,9 @@ Input:
 Output:
 Errors:
 ------------------------------------------------------------------------*/
-uint32 read_32(uint8 *tsp)
+int read_32( uint32 *value, uint8 *tsp)
 {
 	uint8 a,b,c,d;
-	uint32 ret;
 
 	if ((tsp +4) > TNEF_glb.tnef_limit)
 	{
@@ -253,9 +252,9 @@ uint32 read_32(uint8 *tsp)
 	c = *(tsp+2);
 	d = *(tsp+3);
 
-	ret =  long_little_endian(a<<24 | b<<16 | c<<8 | d);
+	*value =  long_little_endian(a<<24 | b<<16 | c<<8 | d);
 
-	return ret;
+	return 0;
 }
 
 /*------------------------------------------------------------------------
@@ -265,10 +264,9 @@ Input:
 Output:
 Errors:
 ------------------------------------------------------------------------*/
-uint16 read_16(uint8 *tsp)
+int read_16( uint16 *value, uint8 *tsp)
 {
 	uint8 a,b;
-	uint16 ret;
 
 	if ((tsp +2) > TNEF_glb.tnef_limit)
 	{
@@ -281,9 +279,9 @@ uint16 read_16(uint8 *tsp)
 	a = *tsp;
 	b = *(tsp + 1);
 
-	ret = little_endian(a<<8 | b);
+	*value = little_endian(a<<8 | b);
 
-	return ret;
+	return 0;
 }
 
 
@@ -298,10 +296,16 @@ Errors:
 char *make_string(uint8 *tsp, int size)
 {
 	static char s[256] = "";
-	int len = (size>sizeof(s)-1) ? sizeof(s)-1 : size;
 
-	strncpy(s,tsp, len);
-	s[len] = '\0';
+	snprintf(s,sizeof(s),"%s",tsp);
+
+	/** 20041106-0929:PLD: Remove this ugly old code and use snprintf() instead 
+	  int len = (size>sizeof(s)-1) ? sizeof(s)-1 : size;
+
+	  strncpy(s,tsp, len);
+	  s[len] = '\0';
+	 **/
+
 	return s;
 }
 
@@ -349,7 +353,7 @@ int handle_props(uint8 *tsp)
 	uint32 x = 0;
 
 
-	num_props = read_32(tsp);
+	read_32(&num_props, tsp);
 	bytes += sizeof(num_props);
 
 	while (x < num_props)
@@ -359,61 +363,61 @@ int handle_props(uint8 *tsp)
 		char filename[256];
 		static int file_num = 0;
 
-		prop_tag = read_32(tsp+bytes);
+		read_32(&prop_tag, tsp+bytes);
 		bytes += sizeof(prop_tag);
 
 		switch (prop_tag & PROP_TYPE_MASK)
 		{
-		case PT_BINARY:
-			num = read_32(tsp+bytes);
-			bytes += sizeof(num);
-			num = read_32(tsp+bytes);
-			bytes += sizeof(num);
-			if (prop_tag == PR_RTF_COMPRESSED)
-			{
-				sprintf (filename, "XAM_%d.rtf", file_num);
-				file_num++;
-				save_attach_data(filename, tsp+bytes, num);
-			}
-			/* num + PAD */
-			bytes += num + ((num % 4) ? (4 - num%4) : 0);
-			break;
-		case PT_STRING8:
-			num = read_32(tsp+bytes);
-			bytes += sizeof(num);
-			num = read_32(tsp+bytes);
-			bytes += sizeof(num);
-			make_string(tsp+bytes,num);
-			bytes += num + ((num % 4) ? (4 - num%4) : 0);
-			break;
-		case PT_UNICODE:
-		case PT_OBJECT:
-			break;
-		case PT_I2:
-			bytes += 2;
-			break;
-		case PT_LONG:
-			bytes += 4;
-			break;
-		case PT_R4:
-			bytes += 4;
-			break;
-		case PT_DOUBLE:
-			bytes += 8;
-			break;
-		case PT_CURRENCY:
-		case PT_APPTIME:
-		case PT_ERROR:
-			bytes += 4;
-			break;
-		case PT_BOOLEAN:
-			bytes += 4;
-			break;
-		case PT_I8:
-			bytes += 8;
-		case PT_SYSTIME:
-			bytes += 8;
-			break;
+			case PT_BINARY:
+				read_32(&num, tsp+bytes);
+				bytes += sizeof(num);
+				read_32(&num, tsp+bytes);
+				bytes += sizeof(num);
+				if (prop_tag == PR_RTF_COMPRESSED)
+				{
+					sprintf (filename, "XAM_%d.rtf", file_num);
+					file_num++;
+					save_attach_data(filename, tsp+bytes, num);
+				}
+				/* num + PAD */
+				bytes += num + ((num % 4) ? (4 - num%4) : 0);
+				break;
+			case PT_STRING8:
+				read_32(&num, tsp+bytes);
+				bytes += sizeof(num);
+				read_32(&num, tsp+bytes);
+				bytes += sizeof(num);
+				make_string(tsp+bytes,num);
+				bytes += num + ((num % 4) ? (4 - num%4) : 0);
+				break;
+			case PT_UNICODE:
+			case PT_OBJECT:
+				break;
+			case PT_I2:
+				bytes += 2;
+				break;
+			case PT_LONG:
+				bytes += 4;
+				break;
+			case PT_R4:
+				bytes += 4;
+				break;
+			case PT_DOUBLE:
+				bytes += 8;
+				break;
+			case PT_CURRENCY:
+			case PT_APPTIME:
+			case PT_ERROR:
+				bytes += 4;
+				break;
+			case PT_BOOLEAN:
+				bytes += 4;
+				break;
+			case PT_I8:
+				bytes += 8;
+			case PT_SYSTIME:
+				bytes += 8;
+				break;
 		}
 		x++;
 	}
@@ -436,25 +440,25 @@ int default_handler(uint32 attribute, uint8 *tsp, uint32 size)
 	uint16 type = ATT_TYPE(attribute);
 
 	switch (type) {
-	case atpTriples:
-		break;
-	case atpString:
-	case atpText:
-		break;
-	case atpDate:
-		break;
-	case atpShort:
-		break;
-	case atpLong:
-		break;
-	case atpByte:
-		break;
-	case atpWord:
-		break;
-	case atpDword:
-		break;
-	default:
-		break;
+		case atpTriples:
+			break;
+		case atpString:
+		case atpText:
+			break;
+		case atpDate:
+			break;
+		case atpShort:
+			break;
+		case atpLong:
+			break;
+		case atpByte:
+			break;
+		case atpWord:
+			break;
+		case atpDword:
+			break;
+		default:
+			break;
 	}
 	return 0;
 
@@ -474,6 +478,7 @@ int read_attribute(uint8 *tsp)
 {
 
 	int bytes = 0, header = 0;
+	int rv = 0;
 	uint32 attribute;
 	uint8 component = 0;
 	uint32 size = 0;
@@ -484,34 +489,32 @@ int read_attribute(uint8 *tsp)
 	//static uint32 attach_loc  = 0; // 2003-02-22-1231-PLD
 	static uint8 *attach_loc  = 0;
 
-	// What component are we look at?
- //
- component = *tsp;
+	component = *tsp;
 
 	bytes += sizeof(uint8);
 
 	// Read the attributes of this component
 
 	if (TNEF_DEBUG) LOGGER_log("%s:%d:TNEF_read_attribute:DEBUG: Reading Attribute...\n",FL);
-	attribute = read_32(tsp+bytes);
-	if (attribute == -1) return -1;
+	rv = read_32(&attribute, tsp+bytes);
+	if (rv == -1) return -1;
 	bytes += sizeof(attribute);
 
 	// Read the size of the information we have to read
 
 	if (TNEF_DEBUG) LOGGER_log("%s:%d:TNEF_read_attribute: Reading Size...\n",FL);
-	size = read_32(tsp+bytes);
-	if (size == -1) return -1;
+	rv = read_32(&size, tsp+bytes);
+	if (rv == -1) return -1;
 	bytes += sizeof(size);
 
 	// The header size equals the sum of all the things we've read
- 	//  so far.
+	//  so far.
 
 	header = bytes;
 
 	// The is a bit of a tricky one [if you're being slow
 	//  it moves the number of bytes ahead by the amount of data of
- 	//  the attribute we're about to read, so that for next
+	//  the attribute we're about to read, so that for next
 	//  "read_attribute()"
 	//  call starts in the right place.
 
@@ -528,144 +531,144 @@ int read_attribute(uint8 *tsp)
 	// --END of ammendment.
 
 	if (TNEF_DEBUG) LOGGER_log("%s:%d:TNEF_read_attribute:DEBUG: Reading Checksum...(offset %d, bytes=%d)\n", FL, tsp -TNEF_glb.tnef_home, bytes);
-	checksum = read_16(tsp+bytes);
+	read_16(&checksum, tsp+bytes);
 	bytes += sizeof(checksum);
 
 	if (TNEF_DEBUG) LOGGER_log("%s:%d:TNEF_read_attribute:DEBUG: Decoding attribute %d\n", FL, attribute);
 
 	switch (attribute) {
-	case attNull:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attFrom:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attSubject:
-		break;
-	case attDateSent:
-		break;
-	case attDateRecd:
-		break;
-	case attMessageStatus:
-		break;
-	case attMessageClass:
-		break;
-	case attMessageID:
-		break;
-	case attParentID:
-		break;
-	case attConversationID:
-		break;
-	case attBody:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attPriority:
-		break;
-	case attAttachData:
-		attach_size=size;
-//		attach_loc =(int)tsp+header; // 2003-02-22-1232-PLD
-		attach_loc =(uint8 *)tsp+header;
-		if (TNEF_glb.savedata && strlen(attach_title)>0 && attach_size > 0) {
-			if (!save_attach_data(attach_title, (uint8 *)attach_loc,attach_size))
-			{
-				if (TNEF_VERBOSE) {
-					if (TNEF_glb.filename_decoded_report == NULL)
-					{
-						LOGGER_log("Decoding: %s\n", attach_title);
-					} else {
-						TNEF_glb.filename_decoded_report( attach_title, (TNEF_glb.verbosity_contenttype>0?"tnef":NULL));
-					}
+		case attNull:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attFrom:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attSubject:
+			break;
+		case attDateSent:
+			break;
+		case attDateRecd:
+			break;
+		case attMessageStatus:
+			break;
+		case attMessageClass:
+			break;
+		case attMessageID:
+			break;
+		case attParentID:
+			break;
+		case attConversationID:
+			break;
+		case attBody:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attPriority:
+			break;
+		case attAttachData:
+			attach_size=size;
+			//		attach_loc =(int)tsp+header; // 2003-02-22-1232-PLD
+			attach_loc =(uint8 *)tsp+header;
+			if (TNEF_glb.savedata && strlen(attach_title)>0 && attach_size > 0) {
+				if (!save_attach_data(attach_title, (uint8 *)attach_loc,attach_size))
+				{
+					if (TNEF_VERBOSE) {
+						if (TNEF_glb.filename_decoded_report == NULL)
+						{
+							LOGGER_log("Decoding: %s\n", attach_title);
+						} else {
+							TNEF_glb.filename_decoded_report( attach_title, (TNEF_glb.verbosity_contenttype>0?"tnef":NULL));
+						}
 
+					}
+				}
+				else
+				{
+					LOGGER_log("%s:%d:TNEF_read_attribute:ERROR: While saving attachment '%s'\n", FL, attach_title);
 				}
 			}
-			else
-			    {
-				LOGGER_log("%s:%d:TNEF_read_attribute:ERROR: While saving attachment '%s'\n", FL, attach_title);
-			}
-		}
-		break;
-	case attAttachTitle:
-		strncpy(attach_title, make_string(tsp+header,size),255);
-		if (TNEF_glb.savedata && strlen(attach_title)>0 && attach_size > 0) {
-			if (!save_attach_data(attach_title, (uint8 *)attach_loc,attach_size))
-			{
-				if (TNEF_VERBOSE) {
-					if (TNEF_glb.filename_decoded_report == NULL)
-					{
-						LOGGER_log("Decoding: %s\n", attach_title);
-					} else {
-						TNEF_glb.filename_decoded_report( attach_title, (TNEF_glb.verbosity_contenttype>0?"tnef":NULL));
-					}
+			break;
+		case attAttachTitle:
+			strncpy(attach_title, make_string(tsp+header,size),255);
+			if (TNEF_glb.savedata && strlen(attach_title)>0 && attach_size > 0) {
+				if (!save_attach_data(attach_title, (uint8 *)attach_loc,attach_size))
+				{
+					if (TNEF_VERBOSE) {
+						if (TNEF_glb.filename_decoded_report == NULL)
+						{
+							LOGGER_log("Decoding: %s\n", attach_title);
+						} else {
+							TNEF_glb.filename_decoded_report( attach_title, (TNEF_glb.verbosity_contenttype>0?"tnef":NULL));
+						}
 
+					}
+				}
+				else
+				{
+					LOGGER_log("%s:%d:TNEF_read_attribute:ERROR: While saving attachment '%s'\n", FL, attach_title);
 				}
 			}
-			else
-			    {
-				LOGGER_log("%s:%d:TNEF_read_attribute:ERROR: While saving attachment '%s'\n", FL, attach_title);
+			break;
+		case attAttachMetaFile:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attAttachCreateDate:
+			break;
+		case attAttachModifyDate:
+			break;
+		case attDateModified:
+			break;
+		case attAttachTransportFilename:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attAttachRenddata:
+			attach_title[0]=0;
+			attach_size=0;
+			attach_loc=0;
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attMAPIProps:
+			handle_props(tsp+header);
+			break;
+		case attRecipTable:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attAttachment:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attTnefVersion:
+			{
+				uint32 version;
+				rv = read_32(&version, tsp+header);
+				if (rv == -1) return -1;
 			}
-		}
-		break;
-	case attAttachMetaFile:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attAttachCreateDate:
-		break;
-	case attAttachModifyDate:
-		break;
-	case attDateModified:
-		break;
-	case attAttachTransportFilename:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attAttachRenddata:
-		attach_title[0]=0;
-		attach_size=0;
-		attach_loc=0;
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attMAPIProps:
-		handle_props(tsp+header);
-		break;
-	case attRecipTable:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attAttachment:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attTnefVersion:
-		{
-			uint32 version;
-			version = read_32(tsp+header);
-			if (version == -1) return -1;
-		}
-		break;
-	case attOemCodepage:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attOriginalMessageClass:
-		break;
-	case attOwner:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attSentFor:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attDelegate:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attDateStart:
-		break;
-	case attDateEnd:
-		break;
-	case attAidOwner:
-		default_handler(attribute, tsp+header, size);
-		break;
-	case attRequestRes:
-		default_handler(attribute, tsp+header, size);
-		break;
-	default:
-		default_handler(attribute, tsp+header, size);
-		break;
+			break;
+		case attOemCodepage:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attOriginalMessageClass:
+			break;
+		case attOwner:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attSentFor:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attDelegate:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attDateStart:
+			break;
+		case attDateEnd:
+			break;
+		case attAidOwner:
+			default_handler(attribute, tsp+header, size);
+			break;
+		case attRequestRes:
+			default_handler(attribute, tsp+header, size);
+			break;
+		default:
+			default_handler(attribute, tsp+header, size);
+			break;
 	}
 	return bytes;
 
@@ -685,6 +688,8 @@ int TNEF_decode_tnef(uint8 *tnef_stream, int size)
 {
 
 	int ra_response;
+	uint32 tnefs;
+	uint16 tnef_attachkey;
 	uint8 *tsp;
 
 	if (TNEF_DEBUG) LOGGER_log("%s:%d:TNEF_decode_tnef:DEBUG: Start. Size = %d\n", FL,size);
@@ -695,23 +700,27 @@ int TNEF_decode_tnef(uint8 *tnef_stream, int size)
 
 	// Read in the signature of this TNEF
 	//
-	if (TNEF_SIGNATURE == read_32(tsp))
+	ra_response = read_32(&tnefs, tsp);
+	if ((ra_response != -1)&&(TNEF_SIGNATURE == tnefs))
 	{
 		if (TNEF_DEBUG) LOGGER_log("%s:%d:TNEF_decode_tnef:DEBUG: TNEF signature is good\n",FL);
 	}
 	else
-	    {
-			 if (TNEF_VERBOSE) LOGGER_log("%s:%d:TNEF_decode_tnef:WARNING: Bad TNEF signature, expecting %lx got %lx\n",FL,TNEF_SIGNATURE,read_32(tsp));
+	{
+		if (TNEF_VERBOSE) LOGGER_log("%s:%d:TNEF_decode_tnef:WARNING: Bad TNEF signature, expecting %lx got %lx\n",FL,TNEF_SIGNATURE,tnefs);
 	}
 
 	// Move tsp pointer along
 	//
 	tsp += sizeof(TNEF_SIGNATURE);
 
-	if (TNEF_DEBUG) LOGGER_log("%s:%d:TNEF_decode_tnef:DEBUG: TNEF Attach Key: %x\n",FL,read_16(tsp));
+	/** Read the TNEF Attach key **/
+	read_16(&tnef_attachkey, tsp);
+	if (TNEF_DEBUG) LOGGER_log("%s:%d:TNEF_decode_tnef:DEBUG: TNEF Attach Key: %x\n",FL,tnef_attachkey);
+
 	// Move tsp pointer along
 	//
-  	tsp += sizeof(uint16);
+	tsp += sizeof(uint16);
 
 	// While we still have more bytes to process,
 	//		go through entire memory block and extract
